/*
 * lab1.c
 */

#include "usart.h"
#include <avr/io.h>
#include <util/delay.h>
#include "DHT.h"

void open_engine() {
    int period;
    const int nmax = 1200;
    /* Make PORTD pins 0, 1, 3, 4 as output */
	DDRC = (1 << 0) | (1 << 1) | (1 << 4) | (1 << 5);	
	period = 2;		/* Set period in between two steps */
		/* Rotate Stepper Motor clockwise with Half step sequence */
    for(int i = 0; i < nmax; ++i)
    {
        PORTC = (1 << 0);
        _delay_ms(period);
        PORTC = (1 << 1);
        _delay_ms(period);
        PORTC = (1 << 2);
        _delay_ms(period);
        PORTC = (1 << 3);
        _delay_ms(period);
    }
}

void close_engine() {
    int period;
    const int nmax = 1200;
    /* Make PORTD pins 0, 1, 3, 4 as output */
	DDRC = (1 << 0) | (1 << 1) | (1 << 4) | (1 << 5);	
	period = 2;		/* Set period in between two steps */
    for(int i = 0; i < nmax; ++i)
    {
    PORTC = 0x09;
	_delay_ms(period);
	PORTC = 0x08;
	_delay_ms(period);
	PORTC = 0x0C;
	_delay_ms(period);
	PORTC = 0x04;
	_delay_ms(period);
	PORTC = 0x06;
	_delay_ms(period);
	PORTC = 0x02;
	_delay_ms(period);
    PORTC = 0x03;
	_delay_ms(period);
	PORTC = 0x01;
	_delay_ms(period);
    }
}

void check_engine(int temp, int hum) {
    static uint8_t engine = -1;
    static uint8_t last_temp = 0;
    static uint8_t last_hum = 0;
    static uint8_t hours = 0;
    static uint8_t minutes = 0;
    static uint8_t seconds = 0;
    const int prag[] = {-5, -7, -10, -15, -20};
    const int delay[] = {3000, 2000, 1000, 500, 300};
    const int TIME_ON = 100;
    int i, val;

	if(++seconds == 60) {
		seconds = 0;
		if(++minutes == 60) {
			minutes = 0;
			if(++hours == 24)
				hours = 0;
		}
	}

    if (engine == -1) {
        last_temp = temp;
        last_hum = hum;
        engine = 0;
        return;
    } else if (engine == 0) {
        //treat engine off case
        for (i = 0; i < 5; ++i) {
            if (temp >= prag[i]) {
                break;
            }
        }
        val = seconds + minutes * 60 + hours * 3600;
        if (prag[i] <= last_temp && val >= delay[i]) {
            seconds = 0;
            minutes = 0;
            hours = 0;
            engine = 1;
            USART0_print("E prea frig, deschid robinetul\r\n");
            open_engine();
        }
    } else if (engine == 1) {
        val = seconds + minutes * 60;
        if (val > TIME_ON) {
            engine = 0;
            USART0_print("A curs destula apa, inchid robinet\r\n");
            open_engine();
        }
    }
}
static int was_pressed = 0;
void check_button()
{
    // configurare porturi folosite
    DDRB &= ~(1 << PB2);
    PORTB |= (1 << PB2);

    // main loop
    static int8_t PB2_flag = 0;
    // cand PB2 este apasat pornim motorul
    if (!(PINB & (1 << PB2)) && PB2_flag == 0) 
    {
            PB2_flag = 1;
            was_pressed = 1;
            USART0_print("Am apasat pe buton, deschid robinet\r\n");
            open_engine();
    }
    _delay_ms(10);
    if (!(PINB & (1 << PB2)) && PB2_flag == 1) 
    {
            PB2_flag = 0;
            was_pressed = 1;
            USART0_print("Am apasat pe buton, inchid robinet\r\n");
            close_engine();
    }

}

void check_status()
{
    char buff[50];
    double temp[1], hum[1];		//Return values of sensor
    int Temp;
    int Hum, i;
	
	//Read from sensor
    USART0_print("Read!\r\n");
    _delay_ms(10);
	DHT_Read(temp, hum);
		
	//Check status
	switch (DHT_status())
	{
		case (DHT_Ok):               
            Temp = (int) hum[0];
            Hum = (int) temp[0];
            sprintf(buff, "Salut %d %d !\r\n", Temp, Hum);
                USART0_print(buff);
                if (Temp < 0) 
                    check_engine(Temp, Hum);
				break;
			case (DHT_Error_Checksum):
				//Do something
                USART0_print("DHT_Error_Checksum\r\n");
				break;
			case (DHT_Error_Timeout):
				//Do something else
                USART0_print("DHT_Error_Timeout\r\n");
				break;
		}
				
		//Sensor needs 1-2s to stabilize its readings
        USART0_print("Check button!\r\n");
		for (i = 0; i < 1000; ++i) {
            check_button();
            if (was_pressed == 1) {
                was_pressed=0;
                break;
            }
        }
        _delay_ms(10);
}


int main()
{
    USART0_init();
    DHT_Setup();
    while (1) {    
        check_button();
        check_status();
    }
    return 0;
}

